@@ -5,7 +5,7 @@ import json as unshadowed_json  # some methods take a named parameter called jso
 from base64 import b64encode
 from urllib.parse import urlparse, urlunparse
 from ssl import SSLError
-from timeit import default_timer
+import time
 
 from http.cookiejar import CookieJar
 
@@ -64,10 +64,11 @@ def insecure_ssl_context_factory():
 class FastHttpSession:
     auth_header = None
 
-    def __init__(self, environment: Environment, base_url: str, insecure=True, **kwargs):
+    def __init__(self, environment: Environment, base_url: str, user: "FastHttpUser", insecure=True, **kwargs):
         self.environment = environment
         self.base_url = base_url
         self.cookiejar = CookieJar()
+        self.user = user
         if insecure:
             ssl_context_factory = insecure_ssl_context_factory
         else:
@@ -94,7 +95,7 @@ class FastHttpSession:
             self.auth_header = _construct_basic_auth_str(parsed_url.username, parsed_url.password)
 
     def _build_url(self, path):
-        """ prepend url with hostname unless it's already an absolute URL """
+        """prepend url with hostname unless it's already an absolute URL"""
         if absolute_http_url_regexp.match(path):
             return path
         else:
@@ -127,6 +128,7 @@ class FastHttpSession:
         auth=None,
         json: dict = None,
         allow_redirects=True,
+        context: dict = {},
         **kwargs,
     ):
         """
@@ -159,12 +161,18 @@ class FastHttpSession:
         # prepend url with hostname unless it's already an absolute URL
         url = self._build_url(path)
 
+        start_time = time.perf_counter()
+
+        if self.user:
+            context = {**self.user.context(), **context}
+
         # store meta data that is used when reporting the request to locust's statistics
-        request_meta = {}
-        # set up pre_request hook for attaching meta data to the request object
-        request_meta["method"] = method
-        request_meta["start_time"] = default_timer()
-        request_meta["name"] = name or path
+        request_meta = {
+            "request_type": method,
+            "name": name or path,
+            "context": context,
+            "exception": None,
+        }
 
         headers = headers or {}
         if auth:
@@ -187,6 +195,7 @@ class FastHttpSession:
 
         # send request, and catch any exceptions
         response = self._send_request_safe_mode(method, url, payload=data, headers=headers, **kwargs)
+        request_meta["response"] = response
 
         if not allow_redirects:
             self.client.redirect_resonse_codes = old_redirect_response_codes
@@ -194,47 +203,31 @@ class FastHttpSession:
         # get the length of the content, but if the argument stream is set to True, we take
         # the size from the content-length header, in order to not trigger fetching of the body
         if stream:
-            request_meta["content_size"] = int(response.headers.get("content-length") or 0)
+            request_meta["response_length"] = int(response.headers.get("response_length") or 0)
         else:
             try:
-                request_meta["content_size"] = len(response.content or "")
+                request_meta["response_length"] = len(response.content or "")
             except HTTPParseError as e:
-                request_meta["response_time"] = int((default_timer() - request_meta["start_time"]) * 1000)
-                self.environment.events.request_failure.fire(
-                    request_type=request_meta["method"],
-                    name=request_meta["name"],
-                    response_time=request_meta["response_time"],
-                    response_length=0,
-                    exception=e,
-                )
+                request_meta["response_time"] = (time.perf_counter() - start_time) * 1000
+                request_meta["response_length"] = 0
+                request_meta["exception"] = e
+                self.environment.events.request.fire(**request_meta)
                 return response
 
         # Record the consumed time
         # Note: This is intentionally placed after we record the content_size above, since
         # we'll then trigger fetching of the body (unless stream=True)
-        request_meta["response_time"] = int((default_timer() - request_meta["start_time"]) * 1000)
+        request_meta["response_time"] = int((time.perf_counter() - start_time) * 1000)
 
         if catch_response:
-            response.locust_request_meta = request_meta
-            return ResponseContextManager(response, environment=self.environment)
+            return ResponseContextManager(response, environment=self.environment, request_meta=request_meta)
         else:
             try:
                 response.raise_for_status()
             except FAILURE_EXCEPTIONS as e:
-                self.environment.events.request_failure.fire(
-                    request_type=request_meta["method"],
-                    name=request_meta["name"],
-                    response_time=request_meta["response_time"],
-                    response_length=request_meta["content_size"],
-                    exception=e,
-                )
-            else:
-                self.environment.events.request_success.fire(
-                    request_type=request_meta["method"],
-                    name=request_meta["name"],
-                    response_time=request_meta["response_time"],
-                    response_length=request_meta["content_size"],
-                )
+                request_meta["exception"] = e
+
+            self.environment.events.request.fire(**request_meta)
             return response
 
     def delete(self, path, **kwargs):
@@ -322,6 +315,7 @@ class FastHttpUser(User):
             max_redirects=self.max_redirects,
             max_retries=self.max_retries,
             insecure=self.insecure,
+            user=self,
         )
 
 
@@ -417,57 +411,46 @@ class ResponseContextManager(FastResponse):
 
     _manual_result = None
 
-    def __init__(self, response, environment):
+    def __init__(self, response, environment, request_meta):
         # copy data from response to this object
         self.__dict__ = response.__dict__
         self._cached_content = response.content
         # store reference to locust Environment
-        self.environment = environment
+        self._environment = environment
+        self._request_meta = request_meta
 
     def __enter__(self):
         return self
 
     def __exit__(self, exc, value, traceback):
+        # if the user has already manually marked this response as failure or success
+        # we can ignore the default behaviour of letting the response code determine the outcome
         if self._manual_result is not None:
             if self._manual_result is True:
-                self._report_success()
+                self._report_request()
             elif isinstance(self._manual_result, Exception):
-                self._report_failure(self._manual_result)
+                self._request_meta["exception"] = self._manual_result
+                self._report_request()
 
-            # if the user has already manually marked this response as failure or success
-            # we can ignore the default behaviour of letting the response code determine the outcome
             return exc is None
 
         if exc:
             if isinstance(value, ResponseError):
-                self._report_failure(value)
+                self._request_meta["exception"] = value
+                self._report_request()
             else:
                 return False
         else:
             try:
                 self.raise_for_status()
             except FAILURE_EXCEPTIONS as e:
-                self._report_failure(e)
-            else:
-                self._report_success()
+                self._request_meta["exception"] = e
+            self._report_request()
+
         return True
 
-    def _report_success(self):
-        self.environment.events.request_success.fire(
-            request_type=self.locust_request_meta["method"],
-            name=self.locust_request_meta["name"],
-            response_time=self.locust_request_meta["response_time"],
-            response_length=self.locust_request_meta["content_size"],
-        )
-
-    def _report_failure(self, exc):
-        self.environment.events.request_failure.fire(
-            request_type=self.locust_request_meta["method"],
-            name=self.locust_request_meta["name"],
-            response_time=self.locust_request_meta["response_time"],
-            response_length=self.locust_request_meta["content_size"],
-            exception=exc,
-        )
+    def _report_request(self):
+        self._environment.events.request.fire(**self._request_meta)
 
     def success(self):
         """
